/*
 * virtual private network daemon (vpnd)
 *
 * cryptographic stuff (c) 1999 Andreas Steinmetz, ast@domdv.de
 * other code (c) 1999 D.O.M. Datenverarbeitung GmbH, author Andreas Steinmetz
 * (c) 2001,2013 Andreas Steinmetz
 *
 * License:
 * This code is in the public domain (*) under the GNU public license.
 * The copyright holders will however retain their copyright.
 * There is no guarantee for the fitness and usability of this code
 * for any purpose. The author and the copyright holders take no
 * responsibility for any damages caused by the use of this code.
 * Distribution and use of this code is explicitly granted provided
 * that the above header is not modified and the above conditions
 * are met.
 * (*) 'public domain' is used here in the sense of the Wassenaar treaty.
 */

#include "vpnd.h"

/*============================================================================*/
/* net-o-rama: ip over ip (dis)connection functionality                       */
/*============================================================================*/

/*
 * getaddr
 *
 * input: ip - the requested ip or the the hostname if dns lookup is allowd
 *
 * return: the ip or 255.255.255.255 in network byte order
 *
 * This procedure returns the requested ip in network byte order.
 */

WORD32 getaddr(WORD08 *ip)
{
	static WORD08 ip255[]="255.255.255.255";	/* error ip	*/
	struct hostent *h;				/* dns info	*/
	WORD08 result[16];				/* conversion	*/
	WORD08 *ptr=result;				/* conversion	*/
	int i;						/* conversion	*/


	/* if dns lookup is allowed */

	if(dns)
	{
		/* if an ip is given return the ip */

		if(inet_addr((char *)ip)!=inet_addr((char *)ip255))
			return inet_addr((char *)ip);

		/* if 255.255.255.255 is given return the ip */

		if(!strcmp((char *)ip,(char *)ip255))
			return inet_addr((char *)ip);

		/* if dns lookup fails return the error ip */

		if(!(h=gethostbyname((char *)ip)))
			return inet_addr((char *)ip255);

		/* now for the weird part: convert result to string
		   and then back to to number as it seems to be the
		   only way to stay endian independent */

		for(i=0;i<4;*ptr++='.',i++)
		{
			if((WORD08)(h->h_addr[i])>99)*ptr++=
				(WORD08)(h->h_addr[i])/100+'0';
			if((WORD08)(h->h_addr[i])>9)*ptr++=
				((WORD08)(h->h_addr[i])%100)/10+'0';
			*ptr++=(WORD08)(h->h_addr[i])%10+'0';
		}
		*(--ptr)=0;

		/* return the looked up ip */

		return inet_addr((char *)result);
	}

	/* if dns is not given return the ip */

	else return inet_addr((char *)ip);
}

/*
 * getaddr64
 *
 * input: ip - the requested ip or the the hostname if dns lookup is allowed
 *
 * output: result - the IPv6 or IPv4 address and family type
 *
 * return: 0 in case of success, -1 in case of error
 *
 * This procedure returns the requested ip in network byte order and
 * sets the family type accordingly.
 */

int getaddr64(WORD08 *ip,struct sockaddr *result)
{
	struct hostent *h;	/* dns info	*/


	/* if dns lookup is allowed */

	if(dns)
	{
		/* try IPv6 address */

		if(ipv6)if(inet_pton(AF_INET6,(char *)ip,
			&((struct sockaddr_in6 *)result)->sin6_addr)==1)
		{
			result->sa_family=AF_INET6;
			return 0;
		}

		/* try IPv4 address */

		if(inet_pton(AF_INET,(char *)ip,
			&((struct sockaddr_in *)result)->sin_addr)==1)
		{
			result->sa_family=AF_INET;
			return 0;
		}

		/* try IPv6 host name lookup */

		if(ipv6)if((h=gethostbyname2((char *)ip,AF_INET6)))
		{
			result->sa_family=AF_INET6;
			memcpy(&((struct sockaddr_in6 *)result)->sin6_addr,
				h->h_addr,16);
			return 0;
		}

		/* try IPv4 host name lookup */

		if((h=gethostbyname2((char *)ip,AF_INET)))
		{
			result->sa_family=AF_INET;
			memcpy(&((struct sockaddr_in *)result)->sin_addr,
				h->h_addr,4);
			return 0;
		}

		/* signal error */

		return -1;
	}

	/* try IPv6 address */

	if(ipv6)if(inet_pton(AF_INET6,(char *)ip,
		&((struct sockaddr_in6 *)result)->sin6_addr)==1)
	{
		result->sa_family=AF_INET6;
		return 0;
	}

	/* try IPv4 address */

	if(inet_pton(AF_INET,(char *)ip,
		&((struct sockaddr_in *)result)->sin_addr)==1)
	{
		result->sa_family=AF_INET;
		return 0;
	}

	/* signal error */

	return -1;
}

/*
 * one_tcpsocket
 *
 * input: anchor - pointer to vpnd global data
 *	  index  - server mode socket index
 *
 * return: 0 - success
 *	   1 - no socket
 *	   2 - bind failed
 *	   3 - set socket option or socket ioctl failed
 *	   4 - connect/listen failed
 *	   5 - socks protocol error
 *	   6 - socks server connection failure/reject
 *
 * This procedure establishes a tcp socket in client or server mode.
 */

static int one_tcpsocket(VPN *anchor,int index)
{
	int s;			/* socket handle		*/
	union
	{
		struct sockaddr addr;
		struct sockaddr_in a4;
		struct sockaddr_in6 a6;
	} a;			/* connection data		*/
	int x;			/* used for socket ioctl	*/
	socklen_t y;		/* used for socket ioctl	*/
	int err;		/* return code			*/
	struct linger l;	/* delay on closing socket	*/
	fd_set w;		/* used to wait for connect	*/
	struct timeval tv;	/* used to expire connect	*/
	WORD08 socksmsg[22];	/* socks v5 protocol buffer	*/
#ifdef SCTP
	struct sctp_paddrparams ap;/* sctp parameter setup	*/
#endif

	/* debug message */

	ENTER("tcpsocket");

	/* reset return value */

	err=0;

	memset(&a,0,sizeof(a));

	/* get initial bind address to get socket family */

	if(anchor->mode)
	{
		if(anchor->socksip)
		{
			if(getaddr64((unsigned char *)anchor->socksip,&a.addr))
				JUMP("address",err1);
		}
		else if(getaddr64((unsigned char *)anchor->client_ip,&a.addr))
			JUMP("address",err1);
	}
	else if(getaddr64(index?(unsigned char *)anchor->server_ip2:
		(unsigned char *)anchor->server_ip,&a.addr))
		JUMP("address",err1);

	/* create socket, handle errors */

	if(anchor->sctpmode)
	{
#ifdef SCTP
		if((s=socket(a.addr.sa_family,SOCK_STREAM,IPPROTO_SCTP))==-1)
#else
		errno=ENOSYS;
#endif
			JUMP("socket",err1);
	}
	else if((s=socket(a.addr.sa_family,SOCK_STREAM,IPPROTO_TCP))==-1)
		JUMP("socket",err1);

	/* bind socket if port given (allow address reuse), handle errors */

	y=anchor->mode?anchor->client_port:anchor->server_port;

	if(y)
	{
		x=1;
		if(setsockopt(s,SOL_SOCKET,SO_REUSEADDR,&x,sizeof(x)))
			JUMP("setsockopt(SO_REUSEADDR)",err2);

		if(a.addr.sa_family==AF_INET)a.a4.sin_port=htons(y);
		else a.a6.sin6_port=htons(y);
		if(bind(s,&a.addr,sizeof(a)))JUMP("bind",err2);
	}

	/* in sctp mode set up sctp connection parameters */

	if(anchor->sctpmode)
	{
#ifdef SCTP
		y=sizeof(ap);
		memset(&ap,0,sizeof(ap));
		if(getsockopt(s,SOL_SCTP,SCTP_PEER_ADDR_PARAMS,&ap,&y))
			JUMP("getsockopt(SCTP_PEER_ADDR_PARAMS)",err3);

		ap.spp_hbinterval=0;
		ap.spp_pathmaxrxt=5;
		ap.spp_sackdelay=500;
		ap.spp_flags|=SPP_HB_DISABLE|SPP_SACKDELAY_ENABLE;
		ap.spp_flags&=~SPP_HB_ENABLE|SPP_HB_DEMAND|
			SPP_SACKDELAY_DISABLE|SPP_HB_TIME_IS_ZERO;

		if(setsockopt(s,SOL_SCTP,SCTP_PEER_ADDR_PARAMS,&ap,sizeof(ap)))
			JUMP("setsockopt(SCTP_PEER_ADDR_PARAMS)",err3);

		if(anchor->maxseg)if(setsockopt(s,SOL_SCTP,SCTP_MAXSEG,
			&anchor->maxseg,sizeof(anchor->maxseg)))
			JUMP("setsockopt(SCTP_MAXSEG)",err3);
#endif
	}

	/* in tcp mode set maximum segment size if requested */

#ifdef SOL_TCP
	else if(anchor->maxseg)if(setsockopt(s,SOL_TCP,TCP_MAXSEG,
		&anchor->maxseg,sizeof(anchor->maxseg)))
		JUMP("setsockopt(TCP_MAXSEG)",err3);
#elif IPPROTO_TCP
	else if(anchor->maxseg)if(setsockopt(s,IPPROTO_TCP,TCP_MAXSEG,
		&anchor->maxseg,sizeof(anchor->maxseg)))
		JUMP("setsockopt(TCP_MAXSEG)",err3);
#endif

	/* set linger on close time, handle errors */

	l.l_onoff=1;
	l.l_linger=10;
	if(setsockopt(s,SOL_SOCKET,SO_LINGER,&l,sizeof(l)))
		JUMP("setsockopt(SO_LINGER)",err3);

	/*  set send buffer size if requested */

	if(anchor->sendbuf)
	    if(setsockopt(s,SOL_SOCKET,SO_SNDBUF,&(anchor->sendbuf),
		sizeof(anchor->sendbuf)))JUMP("setsockopt(SO_SNDBUF)",err3);

	/* prevent packet coalescence */

	x=1;
	if(!anchor->sctpmode)
	{
#ifdef SOL_TCP
		if(setsockopt(s,SOL_TCP,TCP_NODELAY,&x,sizeof(x)))
			JUMP("setsockopt(TCP_NODELAY)",err3);
#elif IPPROTO_TCP
		if(setsockopt(s,IPPROTO_TCP,TCP_NODELAY,&x,sizeof(x)))
			JUMP("setsockopt(TCP_NODELAY)",err3);
#endif
	}
	else
	{
#ifdef SCTP
		if(setsockopt(s,IPPROTO_SCTP,SCTP_NODELAY,&x,sizeof(x)))
			JUMP("setsockopt(SCTP_NODELAY)",err3);
#endif
	}

	/* set type of service and precedence, if given
	   (if any router actually uses these the precedence
	    is restricted to 'priority' below to prevent
	    chaos - do NOT change this! */

	/* set type of service and precedence if requested */

	if(anchor->ipopts)
	{
		x=((anchor->ipopts)<<2)&0x3c;
#ifdef SOL_IP
		if(setsockopt(s,SOL_IP,IP_TOS,&x,sizeof(x)))
			JUMP("setsockopt(IP_TOS)",err3);
#elif IPPROTO_IP
		if(setsockopt(s,IPPROTO_IP,IP_TOS,&x,sizeof(x)))
			JUMP("setsockopt(IP_TOS)",err3);
#endif
	}

	/* set socket to nonblocking, handle errors */

	x=1;
	if(ioctl(s,FIONBIO,&x))JUMP("ioctl(FIONBIO)",err3);

	/* client mode */

	if(anchor->mode)
	{
		/* connect to peer or socks server, handle errors */

		if(anchor->socksip)
		{
			if(a.addr.sa_family==AF_INET)
				a.a4.sin_port=htons(anchor->socksport);
			else a.a6.sin6_port=htons(anchor->socksport);
		}
		else
		{
			if(getaddr64((unsigned char *)
				anchor->server_ip,&a.addr))JUMP("address",err4);
			if(a.addr.sa_family==AF_INET)
				a.a4.sin_port=htons(anchor->server_port);
			else a.a6.sin6_port=htons(anchor->server_port);
		}

		if(connect(s,&a.addr,sizeof(a)))
		{
			/* if connection has not yet been completed */

			if(errno==EINPROGRESS)
			{
				/* set up socket handle for select call */

				FD_ZERO(&w);
				FD_SET(s,&w);

				/* set up timeout value */

				tv.tv_usec=0;
				tv.tv_sec=anchor->connwait;

				/* signal error, if socket is not writeable on
				   return from select */

				if(select(s+1,NULL,&w,NULL,
					anchor->connwait?&tv:NULL)!=1)
						JUMP("select",err4);

				/* get connect completion code, signal error if
				   not possible */

				y=sizeof(x);
				if(getsockopt(s,SOL_SOCKET,SO_ERROR,&x,&y))
					JUMP("getsockopt",err4);

				/* signal error, if connect did not succeed */

				if(x)GOTO("socket error",err4);
			}

			/* signal error in case of all other errors */

			else JUMP("connect",err4);
		}

		/* make peer handle globally available */

		anchor->peer=s;

		/* in case of connect to socks server */

		if(anchor->socksip)
		{
			/* get actual server address */

			if(getaddr64((unsigned char *)
				anchor->server_ip,&a.addr))
					JUMP("address",err5);

			/* do socks V5 authentication (only no auth possible) */

			socksmsg[0]=5;
			socksmsg[1]=1;
			socksmsg[2]=0;

			if(!dosend(socksmsg,8,anchor))
				GOTO("socks auth request",err5);

			if(!dorecv(socksmsg,2,anchor))
				GOTO("socks auth reply",err5);

			if(socksmsg[0]!=5||socksmsg[1])
				GOTO("socks auth denied",err5);

			/* prepare and send socks connect request */

			socksmsg[0]=5;
			socksmsg[1]=1;
			socksmsg[2]=0;

			if(a.addr.sa_family==AF_INET)
			{
				socksmsg[3]=1;
				memcpy(socksmsg+4,&a.a4.sin_addr,4);
				socksmsg[8]=(WORD08)((anchor->server_port)>>8);
				socksmsg[9]=(WORD08)(anchor->server_port);

				if(!dosend(socksmsg,10,anchor))
					GOTO("socks connect request",err5);
			}
			else
			{
				socksmsg[3]=4;
				memcpy(socksmsg+4,&a.a6.sin6_addr,16);
				socksmsg[20]=(WORD08)((anchor->server_port)>>8);
				socksmsg[21]=(WORD08)(anchor->server_port);

				if(!dosend(socksmsg,22,anchor))
					GOTO("socks connect request",err5);
			}
			
			/* read and process socks reply, handle errors */

			if(!dorecv(socksmsg,4,anchor))
				GOTO("socks connect reply",err5);

			if(socksmsg[0]!=5||socksmsg[1]>8||socksmsg[2])
				GOTO("socks connect reply",err5);

			switch(socksmsg[3])
			{
			case 1:	if(!dorecv(socksmsg+4,6,anchor))
					GOTO("socks connect reply",err5);
				break;

			case 4:	if(!dorecv(socksmsg+4,18,anchor))
					GOTO("socks connect reply",err5);
				break;

			default:GOTO("socks connect reply",err5);
			}

			if(socksmsg[1])GOTO("socks connect failure",err6);
		}
	}

	/* server mode */

	else
	{
		/* make socket listen, handle errors */

		if(listen(s,5))JUMP("listen",err4);

		/* make handle globally available */

		if(index)anchor->server2=s;
		else anchor->server=s;
	}

	/* debug message */

	LEAVE("tcpsocket");

	/* signal success */

	goto err0;

err6:	err++;
err5:	err++;
	if(!anchor->sctpmode)shutdown(anchor->peer,2);
	anchor->peer=-1;
err4:	err++;
err3:	err++;
err2:	err++;
	close(s);
err1:	err++;
err0:	return err;
}

/*
 * tcpsocket
 *
 * input: anchor - pointer to vpnd global data
 *
 * return: 0 - success
 *	   1 - no socket
 *	   2 - bind failed
 *	   3 - set socket option or socket ioctl failed
 *	   4 - connect/listen failed
 *	   5 - socks protocol error
 *	   6 - socks server connection failure/reject
 *
 * This procedure establishes a tcp socket in client or server mode.
 */

int tcpsocket(VPN *anchor)
{
	int err;	/* return code */


	/* if in server mode with two listening addresses */

	if(!anchor->mode&&anchor->server_ip2)
	{
		/* open secondary socket, handle errors */

		if((err=one_tcpsocket(anchor,1)))return err;

		/* open primary socket, handle errors */

		else if((err=one_tcpsocket(anchor,0)))
		{
			close(anchor->server2);
			anchor->server2=-1;
			return err;
		}

		/* signal success */

		else return 0;
	}

	/* open single socket, signal errors */

	else return one_tcpsocket(anchor,0);
}

/*
 * accept_socket
 *
 * input:  anchor - pointer to vpnd global data
 *	   index  - socket index
 *
 * output: addr - the address of the connectiong peer
 *
 * return: 0 - success
 *	   1 - accept of connection failed
 *	   2 - unexpected peer connected
 *
 * This procedure accepts a client connection and
 * checks, that the client is the expected one.
 */

int accept_socket(void *addr,VPN *anchor,int index)
{
	union
	{
		struct sockaddr addr;
		struct sockaddr_in a4;
		struct sockaddr_in6 a6;
	} a;			/* connection data		*/
	union
	{
		struct sockaddr addr;
		struct sockaddr_in a4;
		struct sockaddr_in6 a6;
	} b;			/* connection data		*/
	socklen_t x;		/* used for socket ioctl	*/
	int err;		/* return code			*/


	/* debug message */

	ENTER("accept_socket");

	/* reset return value */

	err=0;

	/* try to accept the connection */

	x=sizeof(a);
	if((anchor->peer=accept(index?anchor->server2:anchor->server,
		(struct sockaddr *)(&a),&x))==-1)JUMP("accept",err1);

	/* return peer info to caller */

	memcpy(addr,&a,sizeof(a));

	/* abort if it's not the expected peer */

	if(!getaddr64((unsigned char *)(anchor->client_ip),&b.addr))
	{
		if(b.addr.sa_family==AF_INET&&!b.a4.sin_addr.s_addr)goto out;
		if(b.addr.sa_family==AF_INET6&&!
			memcmp(b.a6.sin6_addr.s6_addr,in6addr_any.s6_addr,16))
			goto out;
		if(a.addr.sa_family!=b.addr.sa_family)GOTO("illegal peer",err2);
		if(b.addr.sa_family==AF_INET&&(a.a4.sin_port!=b.a4.sin_port||
			a.a4.sin_addr.s_addr!=b.a4.sin_addr.s_addr))
			GOTO("illegal peer",err2);
		if(b.addr.sa_family==AF_INET6&&(a.a6.sin6_port!=b.a6.sin6_port||
			memcmp(a.a6.sin6_addr.s6_addr,
			b.a6.sin6_addr.s6_addr,16)))GOTO("illegal peer",err2);
	}

	/* debug message */

out:	LEAVE("accept_socket");

	/* signal success */

	goto err0;

	/* error handling */

err2:	err++;
	close(anchor->peer);
	anchor->peer=-1;
err1:	err++;
err0:	return err;
}

/*
 * disconnect
 *
 * input: dmode  - 0 for peer socket
 *	         - 1 for server socket
 *	         - 2 for peer and server socket
 *		 - 4 forced close of tcp peer socket
 *	  anchor - pointer to vpnd global data
 *
 *
 * This procedure disconnects the tcp/ip connection to the remote peer
 * or the listening server socket.
 */

void disconnect(int dmode,VPN *anchor)
{
	struct linger l;	/* delay on closing socket */


	/* debug message */

	ENTER("disconnect");

	/* if peer socket and if a connection is established */

	if(!(dmode&1))if(anchor->peer!=-1)
	{
		/* if not in serial line or sctp mode */

		if(!anchor->serialmode&&!anchor->sctpmode)
		{
			/* disable linger on close time  if forced close */

			if(dmode&4)
			{
				l.l_onoff=0;
				l.l_linger=0;
				if(setsockopt(anchor->peer,SOL_SOCKET,
					SO_LINGER,&l,sizeof(l)))
						ERRNO("setsockopt(SO_LINGER)");
			}

			/* send TCP_FIN in case of regular close */

			else if(shutdown(anchor->peer,2))ERRNO("shutdown");

			/* close communication channel */

			if(close(anchor->peer))ERRNO("close");
		}

		/* in sctp mode close communication channel */

		else if(anchor->sctpmode)
		{
			if(close(anchor->peer))ERRNO("close");
		}

		/* in serial line mode close serial line */

		else serialhandler(anchor);

		/* execute linkdown process, if there is any */

		doexec(anchor->linkdown,NULL);

		/* reset handle to no peer */

		anchor->peer=-1;
	}

	/* if server socket */

	if(dmode&3)
	{
		/* if the primary server socket exists */

		if(anchor->server!=-1)
		{
			/* close server socket */

			if(close(anchor->server))ERRNO("close");

			/* mark server socket as closed */

			anchor->server=-1;
		}

		/* if the secondary server socket exists */

		if(anchor->server2!=-1)
		{
			/* close server socket */

			if(close(anchor->server2))ERRNO("close");

			/* mark server socket as closed */

			anchor->server2=-1;
		}
	}

	/* debug message */

	LEAVE("disconnect");
}
